<?php

namespace Ep\Hak;

use Ep\Http\AbstractHttpClientFactory;
use Ep\WordPress\Settings\SettingsProviderInterface;
use Zend\ServiceManager\AbstractFactory\ReflectionBasedAbstractFactory;
use Zend\Stdlib\ArrayUtils;

/**
 * Class ConfigProvider
 *
 * @package Ep\Hak
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-hak
 */
class ConfigProvider implements SettingsProviderInterface
{
    public function __invoke()
    {
        $hakSettings = $this->getDefaults()['ep_hak'];

        if (function_exists('get_option')) {
            $hakSettings = ArrayUtils::merge($hakSettings, get_option('ep_hak', []));
        }

        return [
            'dependencies' => $this->getDependencyConfig(),
            'hak' => $hakSettings,

            'forms' => [
                'settings-hak' => include __DIR__ . '/../forms/settings.php',
                'hak-membership' => include __DIR__ . '/../forms/membership.php',
            ],

            'http_clients' => [
                'http.hak' => [
                    'base_uri' => $hakSettings['base_uri'],
                    'http_errors' => true,
                    'headers' => [
                        'Content-Type' => 'application/json',
                        'Device' => $hakSettings['device'],
                        'Username' => $hakSettings['username'],
                        'Token' => $hakSettings['token'],
                        'Cache-Control' => 'no-cache',
                    ],
                ],
            ],

            'wordpress' => [
                'services' => '1' === $hakSettings['enabled'] ? [
                    'HakCoupons',
                    'HakCart',
                    'HakMyAccount',
                ] : [],
            ],

            'twig' => [
                'template_paths' => [
                    EP_HAK_DIR . '/view',
                ],

                'options' => [
                    // ovo mora biti posebno isključeno u produkciji
                    'debug' => defined('WP_DEBUG') ? WP_DEBUG : true,
                    // ako smo u WordPressu keš spremamo u uploads dir
                    'cache' => function_exists('wp_get_upload_dir') && function_exists('get_option')
                        ? rtrim(wp_get_upload_dir()['basedir'], '/') . '/cache/twig'
                        : '',
                ],
            ],
        ];
    }

    public function getDependencyConfig()
    {
        return [
            'aliases' => [
                'Hak' => HakService::class,
                'HakCart' => Service\CartService::class,
                'HakCoupons' => Service\CouponService::class,
                'HakMyAccount' => Service\MyAccountPage::class,
            ],

            'factories' => [
                HakService::class => HakServiceFactory::class,
                Service\CartService::class => ReflectionBasedAbstractFactory::class,
                Service\CouponService::class => ReflectionBasedAbstractFactory::class,
                Service\MyAccountPage::class => ReflectionBasedAbstractFactory::class,
                'http.hak' => AbstractHttpClientFactory::class,
            ],
        ];
    }

    /**
     * Vraća zadane postavke koje provider pruža.
     *
     * @return array
     */
    public function getDefaults()
    {
        return [
            'ep_hak' => [
                'enabled' => '0',
                'base_uri' => 'https://web2.hak.hr',
                'membership_endpoint' => '/tehnickapomocapi/api/1.0/CheckMemberships',
                'device' => '',
                'username' => '',
                'token' => '',
                'checkButtonLabel' => 'Provjeri!',
                'couponName' => 'HAK popust',
                'discountPercentage' => '',
                'dynamic_discount' => [
                    10021809, 10019698, 10012016, 10066441, 10111220, 10111219, 10023197, 10023199, 10022500, 10023884,
                    10023194, 10023887, 10023888, 10023889, 10023891, 10022502, 10024574, 10024575, 10024576, 10024578,
                    10024579, 10024581, 10059726, 10059727, 10059728, 10025149, 10025150, 10025151, 10025152, 10025153,
                    10025154, 10059729, 10025155, 10073927, 10059733, 10025706, 10025707, 10025708, 10025709, 10090621,
                    10090622, 10110578, 10024582, 10026243, 10026244, 10026246, 10026247, 10026248, 10026249, 10117327,
                    10026771, 10060331, 10110583, 10110579, 10027847, 10027310, 10027851, 10027312, 10118220, 10027313,
                    10027314, 10028367, 10028368, 10028369, 10028370, 10028371, 10061526, 10061527, 10030600, 10032282,
                    10109932, 10032858, 10032289, 10033405, 10032852, 10033407, 10033408, 10034006, 10034007, 10034012,
                    10117710, 10036494, 10036495, 10076119, 10076123, 10076124, 10036493, 10037084, 10037085, 10037679,
                    10037680, 10037086, 10037681, 10037682, 10037683, 10037685, 10012596, 10037091, 10037092, 10038285,
                    10038286, 10038287, 10006391, 10038288, 10038882, 10038883, 10038884, 10038885, 10039498, 10040100,
                    10040717, 10109325, 10041299, 10041304, 10041305, 10041306, 10041893, 10041898, 10041899, 10041900,
                    10041901, 10076802, 10061593, 10063365, 10061596, 10070298, 10033973, 10033976, 10071389, 10070845,
                    10070846, 10070847, 10070851, 10071394, 10071395, 10099807, 10099806, 10100390, 10071934, 10026688,
                    10071938, 10071939, 10071940, 10071941, 10071942, 10071396, 10034603, 10072457, 10072458, 10072459,
                    10072460, 10072461, 10072465, 10117713, 10117714, 10072970, 10072971, 10072972, 10072973, 10072975,
                    10072976, 10072977, 10072978, 10034606, 10034607, 10073496, 10061002, 10073501, 10115550, 10074008,
                    10074009, 10024547, 10074010, 10074011, 10074012, 10074013, 10074014, 10074015, 10074016, 10034609,
                    10074526, 10074527, 10074528, 10026777, 10074529, 10074531, 10074533, 10074534, 10034610, 10075040,
                    10100391, 10075042, 10075043, 10075044, 10075045, 10075046, 10075047, 10075048, 10034611, 10076194,
                    10077524, 10077525, 10077526, 10079464, 10079467, 10079466, 10079465, 10078176, 10080092, 10080093,
                    10080094, 10143288, 10143289, 10011634, 10020931, 10103260, 10040526, 10019665, 10152273, 10122478,
                    10122479, 10019666, 10152274, 10039943, 10103261, 10039939, 10050874, 10152275, 10103262, 10103263,
                    10152276, 10027221, 10050875, 10152277, 10100001, 10100002, 10027220, 10152278, 10100003, 10152279,
                    10152280, 10152281, 10103264, 10139216, 10100004, 10152282, 10152283, 10104868, 10104869, 10152284,
                    10100005, 10100006, 10152285, 10103265, 10092383, 10095873, 10093444, 10099143, 10093447, 10092384,
                    10093443, 10095871, 10095876, 10099144, 10099149, 10095201, 10099148, 10099146, 10099145, 10096545,
                    10096548, 10096549, 10096547, 10096546, 10097207, 10097210, 10097211, 10097212, 10099801, 10073666,
                    10099802, 10073667, 10099803, 10113118, 10074172, 10074173, 10074174, 10074175, 10074176, 10074177,
                    10074178, 10074180, 10002837, 10105485, 10002838, 10077698, 10077699, 10116654, 10142646, 10150745,
                    10027745, 10027746, 10070931, 10140518, 10140523, 10140524, 10141042, 10048368, 10142062, 10142067,
                    10142068, 10097825, 10116802, 10028282, 10127442, 10037517, 10028283, 10028284, 10142659, 10142660,
                    10009998, 10150746, 10150747, 10142662, 10141554, 10011170, 10150749, 10119420, 10119421, 10009141,
                    10040523, 10040524, 10040525, 10040549, 10040550, 10025605, 10020932, 10023002, 10040551, 10025606,
                    10152286, 10152287, 10152288, 10122474, 10122475, 10144315, 10144316, 10152289, 10152290, 10152291,
                    10152292, 10152293, 10152294, 10152295, 10144317, 10144318, 10152296, 10025608, 10152297, 10152298,
                    10039938, 10144319, 10119278, 10025842, 10067937, 10105979, 10003925, 10111669, 10111672, 10111673,
                    10111674, 10057985, 10057986, 10027804, 10151305, 10120552, 10058593, 10151308, 10151309, 10151310,
                    10058595, 10058598, 10058600, 10059195, 10059197, 10059198, 10059803, 10059804, 10059806, 10059807,
                    10120553, 10060407, 10060413, 10116110, 10116111, 10061007, 10061009, 10129506, 10061598, 10112484,
                    10061010, 10121847, 10112297, 10112298, 10112299, 10112300, 10112301, 10112303, 10115552, 10116107,
                    10067542, 10120906, 10112486, 10066977, 10111861, 10069752, 10069755, 10070300, 10132196, 10070307,
                    10006120, 10151312, 10083911, 10124249, 10124250, 10116651, 10073668, 10085761, 10139217, 10086351,
                    10110449, 10034107, 10034109, 10034111, 10034737, 10034738, 10034739, 10035354, 10035359, 10035975,
                    10035980, 10036582, 10116112, 10049752, 10049753, 10049755, 10049758, 10049759, 10003022, 10037176,
                    10111855, 10037771, 10037773, 10037774, 10037775, 10037778, 10038372, 10038373, 10038374, 10114381,
                    10038376, 10038379, 10038985, 10038986, 10038987, 10036590, 10038993, 10066438, 10039594, 10105982,
                    10039597, 10039601, 10040205, 10040206, 10040207, 10040210, 10069115, 10040212, 10040815, 10040816,
                    10040818, 10092821, 10050302, 10050303, 10097829, 10050304, 10050306, 10033893, 10150744, 10041410,
                    10041997, 10041999, 10114382, 10042586, 10042587, 10042589, 10042590, 10113119, 10042591, 10043172,
                    10043173, 10042002, 10043788, 10043791, 10043792, 10043794, 10033896, 10033897, 10033898, 10045101,
                    10045104, 10045105, 10045106, 10045107, 10050850, 10116451, 10115112, 10115117, 10046413, 10115695,
                    10046414, 10046415, 10047074, 10047075, 10046416, 10047076, 10028999, 10029001, 10029005, 10047079,
                    10029584, 10029585, 10029589, 10030154, 10030155, 10030157, 10111854, 10030158, 10046419, 10030714,
                    10030715, 10030717, 10030718, 10030719, 10030720, 10030721, 10031270, 10031271, 10031273, 10031274,
                    10031275, 10093440, 10031833, 10051395, 10031836, 10037179, 10031838, 10031276, 10051396, 10031841,
                    10032402, 10051397, 10032404, 10032405, 10032406, 10051403, 10122084, 10032966, 10113764, 10113120,
                    10032971, 10033522, 10033528, 10033529, 10033530, 10032972, 10034758, 10034760, 10034129, 10034762,
                    10113766, 10035371, 10035372, 10035373, 10035374, 10035376, 10035377, 10035379, 10035992, 10080165,
                    10035993, 10033899, 10111856, 10113767, 10035996, 10035998, 10036607, 10111859, 10036608, 10114385,
                    10037194, 10037196, 10037197, 10037200, 10037201, 10016432, 10037790, 10037795, 10114388, 10038390,
                    10038391, 10038392, 10038393, 10038395, 10038397, 10038398, 10038994, 10038995, 10038999, 10039001,
                    10034523, 10034525, 10051945, 10051950, 10111376, 10039602, 10034528, 10039604, 10039605, 10034529,
                    10120907, 10067579, 10058739, 10058740, 10058741, 10058743, 10058744, 10058745, 10058747, 10067580,
                    10059343, 10098479, 10059344, 10059345, 10059346, 10059349, 10059351, 10067581, 10059946, 10059947,
                    10059948, 10059949, 10036990, 10059951, 10059953, 10045427, 10045428, 10045429, 10045430, 10045433,
                    10045435, 10097827, 10035146, 10035147, 10061152, 10061153, 10035148, 10035149, 10025508, 10061154,
                    10061155, 10061156, 10035153, 10035154, 10035761, 10036376, 10116452, 10134279, 10134280, 10134281,
                    10134282, 10134846, 10135416, 10135421, 10135423, 10135424, 10135977, 10135978, 10135980, 10135985,
                    10136530, 10136533, 10136534, 10136537, 10137088, 10137089, 10137090, 10137091, 10137092, 10137093,
                    10137638, 10142430, 10143053, 10143054, 10143055, 10024546, 10143056, 10143057, 10143058, 10143059,
                    10143060, 10143061, 10143675, 10060554, 10143676, 10143677, 10143678, 10143679, 10143680, 10027808,
                    10027809, 10143681, 10143682, 10144294, 10144295, 10144296, 10144297, 10144298, 10144299, 10144300,
                    10144301, 10144302, 10144921, 10144922, 10144923, 10144924, 10144925, 10144926, 10144927, 10144928,
                    10144929, 10145522, 10145523, 10145524, 10145525, 10145526, 10145527, 10145528, 10145529, 10146124,
                    10146125, 10146126, 10034352, 10146127, 10146128, 10146720, 10146722, 10005183, 10005776, 10060555,
                    10048367, 10060556, 10060557, 10005779, 10005783, 10080168, 10006357, 10006358, 10006359, 10006360,
                    10006361, 10006362, 10006930, 10006931, 10006933, 10006934, 10006935, 10007508, 10007509, 10007510,
                    10008072, 10060558, 10061743, 10061744, 10061746, 10061747, 10061748, 10061749, 10060559, 10112459,
                    10008074, 10034353, 10080166, 10008075, 10034354, 10036975, 10037565, 10110448, 10105322, 10062335,
                    10038167, 10038771, 10008078, 10008079, 10034355, 10008080, 10072405, 10008635, 10008636, 10008637,
                    10009196, 10009198, 10009199, 10067695, 10067696, 10067697, 10067698, 10067699, 10067701, 10067702,
                    10116397, 10121451, 10038773, 10038774, 10150741, 10068242, 10068243, 10068244, 10068245, 10068246,
                    10068247, 10068248, 10066538, 10068796, 10097830, 10068797, 10068798, 10068799, 10068800, 10068801,
                    10068803, 10069348, 10069349, 10069350, 10069351, 10069352, 10069353, 10069354, 10069355, 10069356,
                    10066540, 10069902, 10069903, 10069904, 10069905, 10069906, 10069907, 10069908, 10069909, 10069910,
                    10070452, 10070453, 10070454, 10070455, 10070456, 10070457, 10070458, 10024543, 10024544, 10070459,
                    10070460, 10046731, 10038775, 10038776, 10038778, 10038779, 10116453, 10116454, 10116987, 10116988,
                    10071010, 10071011, 10071014, 10071015, 10046732, 10046733, 10046734, 10046735, 10071552, 10071553,
                    10071554, 10072088, 10072089, 10072090, 10072091, 10072092, 10072093, 10072094, 10035169, 10071555,
                    10072610, 10072612, 10072613, 10072614, 10072615, 10072616, 10072617, 10071556, 10071557, 10071558,
                    10071559, 10073125, 10073126, 10073127, 10073128, 10073129, 10073130, 10073131, 10073651, 10073652,
                    10073653, 10073654, 10073655, 10073657, 10073658, 10095792, 10073132, 10073659, 10073133, 10047385,
                    10047386, 10047387, 10074163, 10074165, 10074166, 10074167, 10074168, 10074169, 10074170, 10039375,
                    10039376, 10074171, 10074677, 10074678, 10074679, 10039378, 10039379, 10078354, 10074680, 10039380,
                    10039381, 10039382, 10039383, 10074681, 10074682, 10074683, 10074684, 10039988, 10040596, 10040598,
                    10040600, 10040601, 10040602, 10040603, 10111763, 10041189, 10111764, 10111765, 10111766, 10111767,
                    10111768, 10111769, 10047389, 10075701, 10075702, 10041190, 10041191, 10041192, 10135874, 10113115,
                    10108426, 10112489, 10113276, 10109627, 10111564, 10111565, 10007600, 10007602, 10007603, 10007604,
                    10007605, 10113759, 10044790, 10008162, 10008164, 10008716, 10008717, 10113121, 10008720, 10008723
                ],
            ],
        ];
    }

    /**
     * Vraća forme za postavke.
     *
     * @return array
     */
    public function getForms()
    {
        return [
            'ep_hak' => 'settings-hak',
        ];
    }

    /**
     * Vraća tabove u control panelu koje ovaj provider definira.
     *
     * @return array
     */
    public function getTabs()
    {
        return [[
            'id' => 'general',
            'optionKeys' => [
                'ep_hak',
            ],
        ]];
    }
}